#ifndef __CLF_CAMERA__
#define __CLF_CAMERA__

#include "clf_geometry.h"

class cCamera{
public:
	clf_vector3D vLocation;
	clf_vector3D vForward;
	clf_vector3D vUp;

	cCamera(){
		// setup a default viewer
		setup(clf_vector3D(0,0,3),        // position viewer
					clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
					clf_vector3D(0,1,0));       // which way is up
	}
	cCamera(cCamera& cp){
		vLocation = cp.vLocation;
		vUp = cp.vUp;
		vForward = cp.vForward;
	}
	~cCamera(){}
	void setup(clf_vector3D& pos, clf_vector3D& lookat, clf_vector3D& up){
		vLocation = pos;
		vForward = lookat-pos;
		vUp = up;
		vForward.normalize();
	}
	void move_to(clf_vector3D& pos){
		vLocation = pos;
	}
	void look_at(clf_vector3D& lookat){
		vForward = lookat-vLocation;
		vForward.normalize();
	}
	void orient(clf_vector3D& up){
		vUp = up;
	}

	void apply();
	void apply_mirror();
	void moveby(float fwdx,float fwdy,float fwdz);
	void rotate(float x, float y, float z);
	void rotate_vec(clf_vector3D& v, float x, float y, float z);
	void rotate_locX(float angle);
	void rotate_locY(float angle);
	void rotate_locZ(float angle);

	clf_matrix4x4 rotation_matrix();


















};

class clf_stage{
public:
	clf_stage(){
		left = bottom = 0;
		near_plane = 1.f;
		far_plane = 1000.f;
		perspective_angle = 30;
	}
	~clf_stage(){
	}

	void resize(int l, int b, int w, int h){
		left = l;
		bottom = b;
		width = w;
		height = h;
	}

	void set_perspective(float angle, float np, float fp){
		perspective_angle = angle;
		near_plane = np;
		far_plane = fp;
	}

	void viewport(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
	}
	void perspective(){
		float w = (float)width;
		float h = (float)height;
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
	}

	void prepare(){
		glViewport (left, bottom, (GLsizei)(width), (GLsizei)(height));				// reset the current viewport
		// glMatrixMode (GL_PROJECTION);										// select the projection matrix
		// glLoadIdentity ();													// reset the projection matrix

		float w = (float)width;
		float h = (float)height;

		// the two paths are exactly the same (internally)
		// in the first we calculate the dimensions of the near plane visible area
		// which requires some elementary but sometimes forgotten trigonometry
		// the second is provided by the utility functions in OpenGL
#if 0
		float r = w / h;
		float persp_angle = perspective_angle/2.f;      // half of the perspective angle goes to one side
		float ys = near_plane*(float)tan(persp_angle*PI/180.f);
		float xs = ys*r;
		glFrustum( -xs, xs, -ys, ys, near_plane, far_plane );
#else
		gluPerspective (perspective_angle, w/h, near_plane, far_plane);	// calculate the aspect ratio of the window
#endif
	}

	int left, bottom;
	int width, height;
	float near_plane, far_plane;
	float perspective_angle;
};

#endif // __CLF_CAMERA__
