#include "stdafx.h"
#include "clf_random.h"
#include "clf_primitives.h"
#include "clf_geometry.h"
#include "clf_camera.h"
#include "clf_resources.h"
#include "clf_light.h"

extern HWND  hWnd;						// Window Handle
extern bool keyDown[256];			// holds true / false for each key

static int m_nFPS = 0;
static int m_nFrames = 0;											// FPS and FPS Counter

cCamera g_cam;
clf_stage g_stage;

GLuint g_tex_box = -1;
GLuint g_tex_floor = -1;

clf_matrix4x4 shadow_matrix;

class clf_box{
	clf_vector3D pos;
	clf_vector3D size;
	float angley;
public:
	clf_box(){
		pos = clf_vector3D(30,20,0);
		size = clf_vector3D(20,20,20);
		angley = 22.5f;
	}
	~clf_box(){
	}
	float height(){
		return size.y;
	}
	void frame_move(float fElapsed){
		angley += 22.5f * fElapsed;
	}
	void render_shadow(GLfloat *shadow_mat){
		// save current drawing state
		glPushAttrib(GL_ALL_ATTRIB_BITS);
			// prepare environment
			clf_disable_textures();
			glDisable(GL_DEPTH_TEST);
			glDisable(GL_LIGHTING);
			// save transformation matrix
			glPushMatrix();

				// Multiply by shadow projection matrix (apply shadow transformation)
				glMultMatrixf(shadow_mat);
				// and now apply the object transformations (translation and rotation)
				glTranslatef(pos.x,pos.y,pos.z);
				glRotatef(angley, 0,1,0);
				// set color to black
				glColor3f(.0f, .0f, .0f);
				// and draw
				drawBox(size.x, size.y, size.z);

			// Restore the projection to normal
			glPopMatrix();
		// restore drawing
		glPopAttrib();
	}
	void render(){
		glColor3f(1.0f, 1.0f, 1.0f);
		glPushMatrix();
			glTranslatef(pos.x,pos.y,pos.z);
			glRotatef(angley, 0,1,0);
			drawBox(size.x, size.y, size.z);
		glPopMatrix();
	}
};

// the floor (aka the reference surface for the shadow)
clf_quad g_floor;
// a simple box object
clf_box g_object;
// a light source
clf_light g_light(GL_LIGHT0);

// this function is called when the program starts to initialize the game
void game_init()
{
	g_tex_box = clf_loadTexture("planks02.tga");
	g_tex_floor = clf_loadTexture("floor.tga");

	g_cam.setup(clf_vector3D(0,50,-400),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up

	glEnable (GL_DEPTH_TEST);        // enable depth test
	glEnable(GL_COLOR_MATERIAL);     // use color for drawing the object

	// create and position a light source
	g_light.set_ambient(0.0f, 0.0f, 0.0f, 1.f);
	g_light.set_diffuse(0.9f, 0.9f, 0.9f, 1.f);
	g_light.set_specular(1.f, 1.f, 1.f, 1.f);
	g_light.set_position(-100.f, 100.f, 100.f, 0.f);

	// position a floor (a reference surface for the shadow)
	g_floor.set_vertices(clf_vector3D(-100,-g_object.height()/2,-100), 
												clf_vector3D(-100,-g_object.height()/2,100), 
												clf_vector3D(100,-g_object.height()/2,100), 
												clf_vector3D(100,-g_object.height()/2,-100));
	// calculate the transformation that the light generates shadows on this surface
	shadow_matrix = g_floor.create_shadow_matrix(g_light.position());

	//clf_disable_textures();
}

// this function is called when the window is resized
void resize_window (int width, int height)			// reshape the window when it's moved or resized
{
	g_stage.resize(0,0,width, height);

	glMatrixMode (GL_PROJECTION);									// select the projection matrix
	glLoadIdentity ();														// reset the projection matrix

	// viewport and perspective
	g_stage.prepare(); 
}

// this function is called upon exit to clear allocated resources
void game_terminate()
{
	// release the textures
	glDeleteTextures(1, &g_tex_box);
	glDeleteTextures(1, &g_tex_floor);
}

// the core of the game. This is called repeatedly to update the state of the game
// fElapsed: is the elapsed time since last call. The time is in seconds (usualy a very small number)
void frame_move(float fElapsed)
{
	if (keyDown[VK_ESCAPE]) 
		DestroyWindow(hWnd);

	g_object.frame_move(fElapsed);

	// frame counting mechanism
	// the higher the frame rate the faster the system
	// if frame rate drops below 30 we are in deep trouble
	// we should either optimize the program or buy a new computer
	static dFloat tot = 0;
	tot += fElapsed;
	m_nFrames++;												// Increment Our frame Counter
	if (tot >= 1.f)
	{
		tot = 0;
		m_nFPS = m_nFrames;
		m_nFrames = 0;

		char txt[200];
		sprintf(txt, "GusOnGames, fps:%d",m_nFPS);
		SetWindowText(hWnd, txt);
	}
}

// the drawing function
// it is called right after the frame_move function
void frame_render()
{
	// clear the screen
	glClearColor (0.f, 0.f, 0.2f, 1.f);				// background colour
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and depth buffer

	glMatrixMode (GL_MODELVIEW);								// select the modelview matrix
	glLoadIdentity ();													// reset

	// position the viewer
	g_cam.apply();
	
	// position the light
	g_light.enable();
	g_light.apply();

	// draw the floor
	glColor3f(1.0f, 1.0f, 1.0f);
	clf_enable_textures(g_tex_floor);
	g_floor.render();

	// draw the object shadow
	g_object.render_shadow((GLfloat *)shadow_matrix);
	// draw the object
	clf_enable_textures(g_tex_box);
	g_object.render();

	// draw the light source
	clf_disable_textures();
	clf_light::disable();  // we do not want light effect on the light source
	g_light.draw_source();
}
